/*
 * Written by Oron Peled <oron@actcom.co.il>
 *
 * Some code snippets adapted from spak
 * (http://www.xenos.net/software/spak/)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include "config.h"
#include "linkloop.h"

#define	MAX_IFACES	20

struct if_flag {
	short int flag;
	char *flag_name;
} if_flags[] = {
	{ IFF_UP, "UP" },
	{ IFF_BROADCAST, "BROADCAST" },
	{ IFF_DEBUG, "DEBUG" },
	{ IFF_LOOPBACK, "LOOPBACK" },
	{ IFF_POINTOPOINT, "POINTOPOINT" },
	{ IFF_NOTRAILERS, "NOTRAILERS" },
	{ IFF_RUNNING, "RUNNING" },
	{ IFF_NOARP, "NOARP" },
	{ IFF_PROMISC, "PROMISC" },
	{ IFF_ALLMULTI, "ALLMULTI" },
	{ IFF_MASTER, "MASTER" },
	{ IFF_SLAVE, "SLAVE" },
	{ IFF_MULTICAST, "MULTICAST" },
	{ IFF_PORTSEL, "PORTSEL" },
	{ IFF_AUTOMEDIA, "AUTOMEDIA" },
};

short int get_ifflags(int sock, const char name[]) {
	struct ifreq ifr;

	strncpy(ifr.ifr_name, name, IF_NAMESIZE - 1);
	if (ioctl(sock, SIOCGIFFLAGS, &ifr) < 0) {
		perror("ioctl(SIOCGIFFLAGS)");
		exit(1);
	}
	return ifr.ifr_flags;
}

int main() {
	struct ifconf ifc;
	struct ifreq ifr_x[MAX_IFACES];
	int sock, err;
	u_int8_t myMAC[IFHWADDRLEN];

	if ((sock = socket(PF_PACKET, SOCK_PACKET, 0)) < 0) {
		perror("socket");
		exit(1);
	}

	ifc.ifc_len = MAX_IFACES * sizeof(struct ifreq);
	ifc.ifc_req = ifr_x;
	if((err = ioctl(sock, SIOCGIFCONF, &ifc)) < 0) {
		perror("ioctl");
		exit(1);
	}
	printf("retrieved info for %i interface(s)\n", ifc.ifc_len / sizeof(struct ifreq));
	for (err = 0; err < ifc.ifc_len / sizeof(struct ifreq); err++) {
		int i;
		short int flags = get_ifflags(sock, ifr_x[err].ifr_name);
		get_hwaddr(sock, ifr_x[err].ifr_name, myMAC);
		printf("%s\t%s\t<", ifr_x[err].ifr_name, mac2str(myMAC));
		for(i = 0; i < sizeof(if_flags)/sizeof(if_flags[0]); i++)
			if(flags & if_flags[i].flag)
				printf("%s ", if_flags[i].flag_name);
		putchar('>');
		putchar('\n');
	}
	return EXIT_SUCCESS;
}
